@@ -290,7 +290,7 @@ within their associated applications, if this last ones exist.
     if not isOneFile and not isDir(path):
         raise OsUseError(
             "The following path does not point to one existing file " \
-            "or directory.\n\t''{0}''".format(path)
+            "or directory.\n\t<< {0} >>".format(path)
         )
 
 # Each OS has its own method.
@@ -317,7 +317,7 @@ within their associated applications, if this last ones exist.
 # Unknown method...
     else:
         raise OsUseError(
-            "The opening of file on ''{0}'' OS is not "
+            "The opening of file on << {0} >> OS is not "
             "supported.".format(osName)
         )
 
@@ -439,7 +439,7 @@ This function copy the file having the path ``source`` to the destination path
 
     else:
         raise OsUseError(
-            "The following path points nowhere.\n\t''{0}''".format(source)
+            "The following path points nowhere.\n\t<< {0} >>".format(source)
         )
 
 def destroy(path):
@@ -720,7 +720,7 @@ This function uses the following variables.
     """
 # Directories to keep ?
     if not keepDir in [None, "minimal", "Minimal", "all", "All"]:
-        raise OsUseError("Illegal value of the variable ''keepDir''.")
+        raise OsUseError("Illegal value of the variable << keepDir >>.")
 
     dirMustBeKept = bool(keepDir in ["minimal", "Minimal", "all", "All"])
 
@@ -849,7 +849,7 @@ list returned by ``listFile``.
     return [p for p in nextFile(*args, **kwargs)]
 
 
-class DirView(object):
+class DirView:
     """
 -----------------
 Small description
