@@ -8,8 +8,6 @@ Author    : Christophe BAL
 Mail      : projetmbc@gmail.com
 
 This module contains some simple tools about the Python programming language.
-
-See the documentation for more details.
 """
 
 import gc
@@ -21,6 +19,13 @@ import collections
 # ------------------------- #
 
 class PythonUseError(ValueError):
+    """
+-----------------
+Small description
+-----------------
+
+Base class for errors in the ``python_use`` module of the package ``mistool``.
+    """
     pass
 
 
@@ -30,8 +35,12 @@ class PythonUseError(ValueError):
 
 def dictSingleValues(oneDict):
     """
-This function returns a single value list which contains the values stored in
-one dictionary.
+-----------------
+Small description
+-----------------
+
+This function returns a single value list which contains all the values stored
+in one dictionary.
     """
     return list(set(oneDict.values()))
 
@@ -44,10 +53,14 @@ QUOTE_SYMBOLS = ["'", '"']
 
 def __escape__(data):
     """
+-----------------
+Small description
+-----------------
+
 This small function escapes all the characters that must be escaped in one
 python-like string.
     """
-    if type(data) == str:
+    if isinstance(data, str):
         return data.replace('\\', '\\\\')
 
     else:
@@ -58,12 +71,17 @@ def quote(
     symbol = "'"
 ):
     """
+-----------------
+Small description
+-----------------
+
 This function put the content of the string variable ``text`` into quotes and
 escapes the eventual quote symbols in ``text``.
 
 This function has one optional variable ``symbol`` which indicates the prefered
 quoting symbol which is the single quote ``'`` by default.
 
+
 This function uses the global constant ``QUOTE_SYMBOLS`` which is defined by
 ``QUOTE_SYMBOLS = ["'", '"']``. This indicates the list of the possible quoting
 symbols.
@@ -89,6 +107,10 @@ to ``'another \'example\' to "see" more'``.
 
 def lambdify(text):
     """
+-----------------
+Small description
+-----------------
+
 This function simply produces lambda expressions only from their definitions 
 given via one string.
     """
@@ -99,6 +121,14 @@ def __isGoodType(
     objType,
     wantedType
 ):
+    """
+-----------------
+Small description
+-----------------
+
+This function is used by ``pyRepr`` to verify that the type of the object to
+represent is equal to the type wanted.
+    """
     if objType != wantedType:
         messageType = {
             str : 'string',
@@ -108,9 +138,11 @@ def __isGoodType(
 
         wantedType = messageType.get(wantedType, str(wantedType))
 
+        message = 'With << kind = "{0}" >>, the object to represent ' \
+                + 'must be one << {1} >>.'
+
         raise PythonUseError(
-            'With ''kind = "{0}"'', the object to repr '
-            'must be one {1}.'.format(kind, wantedType)
+            message.format(kind, wantedType)
         )
 
 def pyRepr(
@@ -121,12 +153,21 @@ def pyRepr(
     indent = 0
 ):
     """
+-----------------
+Small description
+-----------------
+
 This function returns one string representation, usable in ¨python codes, of one
 ¨python object.
 
-There are five variables. See the documentation for more explanations.
 
-    1) ``obj`` is the python object to print in one ¨python code.
+-------------
+The arguments
+-------------
+
+This function uses five variables.
+
+    1) ``obj`` is the ¨python object to print in one ¨python code.
 
     2) ``kind`` is an optional argument to indicate the kind of object. The
     default value is ``None`` which asks to use the type of the object. The
@@ -135,8 +176,7 @@ There are five variables. See the documentation for more explanations.
         * ``"bool"``, ``"dict"``, ``"float"``, ``"int"``, ``"list"`` and
         ``"str"`` indeed corresponds to ¨python types supported.
 
-        * ``"regex"`` is useful for pattern definied using
-        ``re.compile("...")``.
+        * ``"regex"`` is useful for pattern definied via ``re.compile("...")``.
 
         * ``"repr"`` simply asks to use the standard ¨python representation.
 
@@ -231,6 +271,10 @@ def __reprList(
     depth
 ):
     """
+-----------------
+Small description
+-----------------
+
 This function is for writing lists.
     """
     if depth == 0:
@@ -262,6 +306,10 @@ def __reprDict(
     depth
 ):
     """
+-----------------
+Small description
+-----------------
+
 This function is for writing dictionaries.
     """
     try:
