@@ -10,6 +10,8 @@ Mail      : projetmbc@gmail.com
 This script contains some useful functions for manipulating strings.
 """
 
+from mistool import regex_use
+
 
 # ------------------------- #
 # -- FOR ERRORS TO RAISE -- #
@@ -39,15 +41,27 @@ def replace(
 Small description
 -----------------
 
-This function replaces in the argument ``text`` using the associations defined
-in the dictionary ``replacement``. Here is a small example.
+This function does replacements in the argument ``text`` using the associations
+defined in the dictionary ``replacement``.
+
+
+warning::
+    The function does the replacements sequentially from the longer word to the
+    shorter one.
+
+
+Here is a small example.
 
 python::
     from mistool import string_use
 
     littleExample = string_use.replace(
             text        = "One, two, three,..."
-            replacement = {'One': "1", 'Two': "2", 'Three': "3"}
+            replacement = {
+                'One'  : "1",
+                'Two'  : "2",
+                'Three': "3"
+            }
         )
     )
 
@@ -55,6 +69,12 @@ python::
 In that code, ``littleExample`` is equal to ``"1, 2, 3,..."``.
 
 
+info::
+    This function has not been build for texts to be replaced that contains some
+    other texts to also replace. If you need this kind of feature, take a look
+    at the class ``MultiReplace``.
+
+
 -------------
 The arguments
 -------------
@@ -64,17 +84,267 @@ This function uses the following variables.
     1) ``text`` is a string argument corresponding to the text where the
     replacements must be done.
 
-    2) ``replacement`` is a dictionary where each couple ``(key, value)`` is
-    a couple ``(text to find, replacement)``.
+    2) ``replacement`` is a dictionary where each couple ``(key, value)`` is of
+    the kind ``(text to find, replacement)``.
     """
-    for old in sorted(
+    longToShortKeys = sorted(
         replacement.keys(),
         key = lambda t: -len(t)
-    ):
+    )
+
+    for old in longToShortKeys:
         text = text.replace(old, replacement[old])
 
     return text
 
+PATTERN_GROUP_WORD = regex_use.PATTERN_GROUP_WORD
+
+class MultiReplace:
+    """
+-----------------
+Small description
+-----------------
+
+The purpose of this class is to replace texts that can contain some other texts
+to also be replaced. Here is an example of use.
+
+python::
+    form mistool import string_use
+
+    myReplace = string_use.MultiReplace(
+        replacement = {
+            'W1' : "word #1",
+            'W2' : "word #2",
+            'W12': "W1 and W2"
+        },
+        pattern = string_use.PATTERN_GROUP_WORD["var"]
+    )
+
+    print(myReplace.replace("W1 and W2 = W12"))
+
+
+Launched in a terminal, the preceding code will produce the following output.
+
+terminal::
+    word #1 and word #2 = word #1 and word #2
+
+
+The only technical thing is the use of ``string_use.PATTERN_GROUP_WORD["var"]``
+which is a regex grouping pattern. You can use directly the following patterns
+or use your own grouping pattern.
+
+    1) ``string_use.PATTERN_GROUP_WORD["en"]`` is for words only made of ¨ascii
+    letters.
+
+    2) ``string_use.PATTERN_GROUP_WORD["fr"]`` is for words only made of ¨ascii
+    letters and the special letters "â", "à", "é", "è", "ê", "ë", "î", "ï", "ô",
+    "ù", "ü", and "ç".
+
+    2) ``string_use.PATTERN_GROUP_WORD["var"]`` is for words only starting with
+    one ¨ascii letter followed eventually by other ¨ascii letters, digits and
+    underscores.
+
+
+warning::
+    Before doing the replacements in a text, the class first build the
+    dictionary ``self.directReplacement`` for which the replacements has been changed so to not contain text to be replaced. With the preceeding code,
+    this dictionary is equal to the following one.
+
+    python::
+        {
+            'W1' : "word #1",
+            'W2' : "word #2",
+            'W12': "word #1 and word #2"
+        }
+
+
+info::
+    In the following dictionnary defining replacements, there are cyclic
+    definitions. In that case, the class will raise an error.
+
+    python::
+        replacement = {
+            'WRONG_1': "one small text and  WRONG_2",
+            'WRONG_2': "one small text, and then WRONG_3",
+            'WRONG_3': "with WRONG_1, there is one problem here"
+        }
+
+
+-------------
+The arguments
+-------------
+
+The instanciation of this class uses the following variables.
+
+    1) ``replacement`` is a dictionary where each couple ``(key, value)`` is of
+    the kind ``(text to find, replacement)``. Here the replacements can contain
+    also text to be found.
+
+    2) ``pattern`` is a regex grouping pattern indicating the kind of words to
+    be replaced. By default, ``pattern = PATTERN_GROUP_WORD["en"]`` where
+    ``PATTERN_GROUP_WORD`` is a renaming of ``regex_use.PATTERN_GROUP_WORD``
+    (see the module ``regex_use`` for more informations).
+    """
+
+    def __init__(
+        self,
+        replacement,
+        pattern = PATTERN_GROUP_WORD["en"]
+    ):
+        self.pattern           = pattern
+        self.replacement       = replacement
+        self.directReplacement = None
+
+        self.__crossReplace = None
+        self.__oldWords     = None
+
+        self.__update()
+
+    def __update(self):
+        """
+-----------------
+Small description
+-----------------
+
+This method simply launches methods so as to verify that there is no cyclic
+replacements, and then to build the direct replacements dictionary
+``self.directReplacement``.
+        """
+        self.__crossReplace = {}
+
+        self.__noCycle()
+
+        self.__replaceInReplacement()
+
+    def __noCycle(self):
+        """
+-----------------
+Small description
+-----------------
+
+This method verifies that there is no cyclic replacements.
+        """
+# Building the crossing replacements.
+        self.__oldWords = list(self.replacement.keys())
+
+        for old, new in self.replacement.items():
+            self.__crossReplace[old] = [
+                x for x in self.pattern.findall(new)
+                if x in self.__oldWords
+            ]
+
+        self.__viciousCircles()
+
+    def __viciousCircles(
+        self,
+        visitedWord = [],
+        nextWord    = None
+    ):
+        """
+-----------------
+Small description
+-----------------
+
+This method does all the major job for ``self.__noCycle``.
+        """
+        if nextWord == None:
+            nextWord = self.__oldWords
+
+        for old in nextWord:
+            oldInNew = self.__crossReplace[old]
+
+            if old in oldInNew:
+                raise StringUseError(
+                    "<< {0} >> is used in its associated replacement." \
+                        .format(old)
+                )
+
+            elif old in visitedWord:
+                pos = visitedWord.index(old)
+
+                visitedWord = visitedWord[pos:]
+                visitedWord.append(old)
+                visitedWord = [""] + visitedWord
+
+                raise StringUseError(
+                    "The following viscious circle has been found." \
+                    + "\n\t + ".join(visitedWord)
+                )
+
+            else:
+                for new in oldInNew:
+                    self.__viciousCircles(
+                        visitedWord = visitedWord + [old],
+                        nextWord    = self.__crossReplace[old]
+                    )
+
+    def __replaceInReplacement(self):
+        """
+-----------------
+Small description
+-----------------
+
+This method builds ``self.directReplacement`` the direct replacements
+dictionary.
+        """
+        self.directReplacement = {}
+
+        for old, new in self.replacement.items():
+            self.directReplacement[old] = self.__replace(new)
+
+    def __replace(
+        self,
+        text
+    ):
+        """
+-----------------
+Small description
+-----------------
+
+This method does all the major job for ``self.__replaceInReplacement``.
+        """
+        if not text:
+            return text
+
+        newText = ""
+
+        while True:
+# Source :
+#    http://www.developpez.net/forums/d958712/autres-langages/python-zope/general-python/amelioration-split-evolue/#post5383767
+            newText = self.pattern.sub(self.__apply, text)
+
+            if newText == text:
+                return text
+
+            else:
+                text = newText
+
+    def __apply(
+        self,
+        match
+    ):
+        """
+-----------------
+Small description
+-----------------
+
+This method is used to does the replacements corresponding the matching groups.
+        """
+        return self.replacement.get(match.group(1), match.group(0))
+
+    def replace(
+        self,
+        text
+    ):
+        """
+This method does the replacements in one text (indeed it simply calls the
+function ``replace`` with the attribut ``self.directReplacement``).
+        """
+        return replace(
+            text        = text,
+            replacement = self.directReplacement
+        )
+
 
 # ----------------- #
 # -- LOOKING FOR -- #
@@ -82,7 +352,7 @@ This function uses the following variables.
 
 # Source : http://www.developpez.net/forums/d921494/autres-langages/python-zope/general-python/naturel-chaine-stockage
 
-class AutoComplete(object):
+class AutoComplete:
     """
 -----------------
 Small description
@@ -122,7 +392,8 @@ terminal::
 
 
 The method ``matching`` simply gives all the words starting with the prefix
-given.
+given. If the prefix is empty, the matching words are all the words defining the
+auto completion.
 
 This search indeed uses a special "magical" dictionary which is stored in the
 attribut ``dict``. Here ``myAC.dict`` is equal to the dictionary above where the
@@ -182,7 +453,7 @@ There is two other useful methods (see their docstrings for more informations).
     1) ``build`` simply builds the "magical" dictionary. This method can be used
     for local updating of the list of words used for the auto completion.
 
-    1) ``missing`` simply gives the letters remaining after one prefix in a
+    2) ``missing`` simply gives the letters remaining after one prefix in a
     word.
 
 
@@ -190,18 +461,18 @@ There is two other useful methods (see their docstrings for more informations).
 The arguments
 -------------
 
-This class uses the following variables.
+The instanciation of this class uses the following variables.
 
     1) ``listWord`` is the list of words to use for the auto completions.
 
-    1) ``depth`` is the minimal size of the prefix used to look for the auto
+    2) ``depth`` is the minimal size of the prefix used to look for the auto
     completions. By default, ``depth = 0`` which indicates to start the auto
     completion with the first letter.
 
     infoo:
         `3` seems to be a good value of ``depth`` for ¨gui application.
 
-    1) ``dictAsso`` is a "magical" dictionary that eases the auto completion.
+    3) ``dictAsso`` is a "magical" dictionary that eases the auto completion.
     This dictionary is build by the method ``build`` from the list of words tio
     be used for the auto completions.
     """
@@ -217,20 +488,15 @@ This class uses the following variables.
                 "You must give either the value of ``listWord`` or ``dict``."
             )
 
-        self.dict  = dict
-        self.depth = depth
+        self.listWord = listWord
+        self.dict     = dict
+        self.depth    = depth
 
 # We have to build the magical dictionary.
         if self.dict == None:
-            self.listWord = listWord
-
             self.build()
 
-    def build(
-        self,
-        listWord = None,
-        depth    = None
-    ):
+    def build(self):
         """
 -----------------
 Small description
@@ -241,49 +507,24 @@ Indeed, if you create a class ``AutoComplete`` without giving a "magical"
 dictionary ``dict``, the method ``build`` is automatically called.
 
 
-If it is necessary, you can use the variable ``listWord``and ``depth`` which
-have the same meaning as for the creation of the class. If you don't use one of
-this argument, then the method will simply use the corresponding one given when
-creating the  the class.
-
-
 info::
     The idea of the magical dictionary comes from cf::``this discusion ;
     http://www.developpez.net/forums/d921494/autres-langages/python-zope/general-python/naturel-chaine-stockage``
-
-
--------------
-The arguments
--------------
-
-This method uses the following variables.
-
-    1) ``listWord`` is the list of words to use for the auto completions. If you
-    don't use it, the method will use the list of words given at the creation of
-    the class.
-
-    1) ``depth`` is the minimal size of the prefix used to look for the auto
-    completions. If you don't use it, the method will use the depth available
-    at the creation of the class.
         """
-        if listWord == None:
-            listWord = self.listWord
-
-        if depth == None:
-            depth = self.depth
-
 # Sorted list of single words.
-        shortSortedList = list(set(listWord))
+        shortSortedList = list(set(self.listWord))
         shortSortedList = sorted(shortSortedList)
 
 # Maximum depth.
+        depth = self.depth
+
         if depth == 0:
             for oneWord in shortSortedList:
                 depth = max(depth, len(oneWord) - 1)
 
 # Let's build the magical dictionary.
         self.dict = {
-            'words'    : shortSortedList,
+            'words'      : shortSortedList,
             'completions': {}
         }
 
@@ -330,7 +571,8 @@ string variable ``prefix``.
 Small description
 -----------------
 
-Giving a word ``"prefixExample"`` and one prefix ``"pre"``, this method will return simply ``"fixExample"``.
+Giving a word ``"prefixExample"`` and one prefix ``"pre"``, this method will
+return simply ``"fixExample"``.
 
 
 -------------
@@ -341,7 +583,7 @@ This method uses the following variables.
 
     1) ``prefix`` is a string corresponding to the prefix expected.
 
-    1) ``word`` is a string where the prefix ``prefix`` must be removed.
+    2) ``word`` is a string where the prefix ``prefix`` must be removed.
         """
         if not word.startswith(prefix):
             raise StringUseError(
@@ -412,6 +654,350 @@ This function uses the following variables.
 
     return ", ".join(listText[:-1]) + " " + andText + " " + listText[-1]
 
+def split(
+    text,
+    sep,
+    escape = None,
+    strip  = False
+):
+    """
+-----------------
+Small description
+-----------------
+
+This function allows to split a text using a list of separators, and not only a
+single separator. Here is an example.
+
+python::
+    from mistool import string_use
+
+    splitText = string_use.split(
+        text  = "p_1 ; p_2 ; p_3 | r_1 ; r_2 | s",
+        sep   = ["|", ";"],
+        strip = True
+    )
+
+In this code, the variable ``splitText`` is equal to the following list :
+``['p_1', 'p_2', 'p_3', 'r_1', 'r_2', 's']``.
+
+
+You can escape separators like in the following example that also uses the
+possibility to give a single string instead of a one value list.
+
+python::
+    from mistool import string_use
+
+    splitText = string_use.split(
+        text   = "p_1 \; p_2 ; p_3",
+        sep    = ";",
+        escape = "\\",
+        strip  = True
+    )
+
+In this code, the variable ``splitText`` is equal to the following list :
+``['p_1 \; p_2', 'p_3']``.
+
+
+-------------
+The arguments
+-------------
+
+This function uses the following variables.
+
+    1) ``text`` is simply the text to split.
+
+    2) ``sep`` is either a string for a single separator, or a list of
+    separators.
+
+    3) ``escape`` is a string used to escape the separators. By default,
+    ``escape = None`` which indicates that there is no escaping feature.
+
+    4) ``strip`` is a boolean variable to strip, or not, each pieces of text
+    found. By default, ``strip = False``.
+"""
+# No separator
+    if not sep:
+        if strip:
+            text = text.strip()
+
+        return [text]
+
+# At least one separator
+    if isinstance(sep, str):
+        sep = [sep]
+
+    if not isinstance(sep, list) \
+    and not isinstance(sep, tuple):
+        raise StringUseError(
+            "The variable << sep >> must be a list or tuple of strings."
+            "\n\t{0}".format(sep)
+        )
+
+    for oneSep in sep:
+        if not isinstance(oneSep, str):
+            raise StringUseError(
+                "The variable << sep >> must be list or tuple of strings."
+                "\n\t{0}".format(sep)
+            )
+
+# << Warning ! >> We must sort the opening symbolic tags from the longer one
+# to the shorter.
+    sorted(sep, key = lambda x: -len(x))
+
+    answer = []
+    iMax   = len(text)
+    i      = 0
+    iLast  = i
+
+    while(i < iMax):
+# The bigger is the winner !
+        for oneSep in sep:
+            if text[i:].startswith(oneSep):
+                lastPiece = text[iLast: i]
+
+                if not(escape and lastPiece.endswith(escape)):
+                    i    += len(oneSep)
+                    iLast = i
+
+                    answer.append(lastPiece)
+
+                    break
+
+        i += 1
+
+    remainingText = text[iLast:]
+
+    if remainingText:
+        answer.append(remainingText)
+
+    if strip:
+        answer = [x.strip() for x in answer]
+
+# The job has been done !
+    return answer
+
+class _SplitKind:
+    """
+-----------------
+Small description
+-----------------
+
+This class is simply an object like class used by the method ``__iter__`` of
+the class ``MultiSplit``.
+    """
+
+    def __init__(
+        self,
+        type,
+        val
+    ):
+        self.type = type
+        self.val  = val
+
+class MultiSplit:
+    """
+-----------------
+Small description
+-----------------
+
+The purpose of this class is to split a text at different levels. You can use
+either a list view version of the split text, or work with a special iterator.
+
+
+Here is an example of the list view version of the split text.
+
+python::
+    from mistool import string_use
+
+    splitText = string_use.MultiSplit(
+        text  = "p_1 , p_2 ; p_3 | r_1 ; r_2 | s",
+        sep   = ["|", (";", ",")],
+        strip = True
+    )
+
+    listView = splitText.listView
+
+
+In this code, the variable ``listView`` is equal to the following list. There
+is as many level of lists that the length of the list ``sep``. The use of
+``(";", ",")`` simply indicates that the separators ``;`` and ``,`` have the
+same importance.
+
+python::
+    [
+        ['p_1', 'p_2', 'p_3'],
+        ['r_1', 'r_2'],
+        ['s']
+    ]
+
+
+Let see with an example how to use the instance as an iterator so as to ease
+the walk in the split text.
+
+python::
+    from mistool import string_use
+
+    splitText = string_use.MultiSplit(
+        text  = "p_1 , p_2 ; p_3 | r_1 ; r_2 | s",
+        sep   = ["|", ";", ","],
+        strip = True
+    )
+
+    for x in splitText:
+        print("{0} ---> {1}".format(x.type, x.val))
+
+
+Launched in a terminal, the code will produce the following output that shows
+how to work easily with the iterator of the class ``MultiSplit``. You have to
+know that the type is a string, and also that for the "sep" type, the associated
+value is equal to the separator used when the instance has been created.
+
+terminal::
+    sep ---> |
+    sep ---> ;
+    sep ---> ,
+    list ---> ['p_1', 'p_2']
+    sep ---> ;
+    sep ---> ,
+    list ---> ['p_3']
+    sep ---> |
+    sep ---> ;
+    sep ---> ,
+    list ---> ['r_1']
+    sep ---> ;
+    sep ---> ,
+    list ---> ['r_2']
+    sep ---> |
+    sep ---> ;
+    sep ---> ,
+    list ---> ['s']
+
+
+-------------
+The arguments
+-------------
+
+The instanciation of this class uses the following variables.
+
+    1) ``text`` is simply the text to split.
+
+    2) ``sep`` is either a string for a single separator, or a list from the
+    stronger separator to the weaker one. You can use list or tuple to gather
+    separators having the same level of priority.
+
+    3) ``escape`` is a string used to escape the separators. By default,
+    ``escape = None`` which indicates that there is no escaping feature.
+
+    4) ``strip`` is a boolean variable to strip, or not, each pieces of text
+    found. By default, ``strip = False``.
+"""
+
+    def __init__(
+        self,
+        text,
+        sep,
+        strip  = False,
+        escape = None
+    ):
+        self.text   = text
+        self.sep    = sep
+        self.strip  = strip
+        self.escape = escape
+
+        if isinstance(self.sep, str):
+            self.sep = [self.sep]
+
+        self.listView = self.build()
+
+    def build(self):
+        """
+-----------------
+Small description
+-----------------
+
+This method builds the list view version of the split text.
+
+Indeed all the job is done by the hidden method ``self.__build``.
+        """
+        return self.__build(
+            text   = self.text,
+            sep    = self.sep,
+            strip  = self.strip,
+            escape = self.escape
+        )
+
+    def __build(
+        self,
+        text,
+        sep,
+        strip,
+        escape
+    ):
+# No separator
+        if not sep:
+            if strip:
+                text = text.strip()
+
+            answer = [text]
+
+# At least one separator
+        else:
+            answer = split(
+                text   = text,
+                sep    = sep[0],
+                escape = escape,
+                strip  = strip
+            )
+
+            remainingSep = sep[1:]
+
+            if remainingSep:
+                answer = [
+                    self.__build(
+                        text   = onePiece,
+                        sep    = remainingSep,
+                        escape = escape,
+                        strip  = strip
+                    )
+                    for onePiece in answer
+                ]
+
+# The job has been done !
+        return answer
+
+    def __iter__(self):
+        return self.__iter(listView = self.listView)
+
+    def __iter(
+        self,
+        listView,
+        depth = 0
+    ):
+        if listView:
+            if isinstance(listView[0], str):
+                yield _SplitKind(
+                    type = "sep",
+                    val  = self.sep[depth]
+                )
+
+                yield _SplitKind(
+                    type = "list",
+                    val  = listView
+                )
+
+            else:
+                for x in listView:
+                    yield _SplitKind(
+                        type = "sep",
+                        val  = self.sep[depth]
+                    )
+
+                    for y in self.__iter(
+                        listView = x,
+                        depth    = depth + 1
+                    ):
+                        yield y
+
 def beforeAfter(
     text,
     start,
@@ -1313,7 +1899,7 @@ This function uses the following variables.
 # All the job has been done.
     return answer
 
-class step():
+class step:
     """
 -----------------
 Small description
